/*
 * FilteredServletContextBeanInfo.java
 *
 * BeanDoc generated on Fri Mar 16 at 12:39:55 EDT 02012.
 * 
 * See FilteredServletContext.java for copyright information.
 */

package org.teatrove.teaservlet.util;

import java.beans.*;
import java.lang.reflect.*;

import java.util.Vector;

/**
 * The BeanInfo class describing FilteredServletContext. 
 *
 * @author Brian S O'Neill
 */
public class FilteredServletContextBeanInfo extends SimpleBeanInfo {

    /** BeanDescriptor returned by the <code>getBeanDescriptor</code> 
        method */
    private BeanDescriptor mBeanDescriptor;

    /** BeanInfo array returned by the <code>getAdditionalBeanInfo</code> 
        method */
    private BeanInfo[] mAdditionalBeanInfo;

    /** MethodDescriptor array returned by the 
        <code>getMethodDescriptors</code> method */
    private MethodDescriptor[] mMethodDescriptors;

    /** PropertyDescriptor array returned by the 
        <code>getPropertyDescriptors</code> method */
    private PropertyDescriptor[] mPropertyDescriptors;
    
    /** The index of the default property */
    private int mDefaultPropertyIndex;


    /**
     * Creates a new FilteredServletContextBeanInfo.
     */
    public FilteredServletContextBeanInfo() {        
                          
        mDefaultPropertyIndex = -1;
        
    }
     
    /**
     * Returns a BeanDescriptor that describes FilteredServletContext.
     */    
    public BeanDescriptor getBeanDescriptor() {
        
        if (mBeanDescriptor == null) {
            mBeanDescriptor = createBeanDescriptor();       
        }

        return mBeanDescriptor;
    }

    /**
     * Returns a BeanInfo array containing BeanInfo object's for:<br>
     * <code>java.lang.Object</code><br>
     * <code>javax.servlet.ServletContext</code><br>      
     */  
    public BeanInfo[] getAdditionalBeanInfo() {

        if (mAdditionalBeanInfo == null) {
            mAdditionalBeanInfo = createAdditionalBeanInfo();       
        }

        return mAdditionalBeanInfo;
    }

    /**    
     * Returns MethodDescriptors that describe the methods of 
     * FilteredServletContext.  Call the 
     * <code>getAdditionalBeanInfo</code> method to get the 
     * method descriptors of the superclass and interfaces.
     */ 
    public MethodDescriptor[] getMethodDescriptors() {

        if (mMethodDescriptors == null) {
            mMethodDescriptors = createMethodDescriptors();       
        }

        return mMethodDescriptors;
    }

    /**    
     * Returns a PropertyDescriptor array that describes the properties of 
     * FilteredServletContext.
     */ 
    public PropertyDescriptor[] getPropertyDescriptors() {

        if (mPropertyDescriptors == null) {
            mPropertyDescriptors = createPropertyDescriptors();     
        }

        return mPropertyDescriptors;
    }

    /**
     * Return the index of the default property.
     */
    public int getDefaultPropertyIndex() {
        return mDefaultPropertyIndex;
    }


    //
    // Non-public interface
    //

    // Create the bean descriptor for the bean.
    private BeanDescriptor createBeanDescriptor() {                        
        
        BeanDescriptor bd = 
            new BeanDescriptor(FilteredServletContext.class);
        
        bd.setName("FilteredServletContext");
        bd.setDisplayName("FilteredServletContext");
        bd.setShortDescription("A ServletContext wrapper that passes all calls to an internal\n ServletContext. This class is designed for subclasses to override or\n hook into the behavior of a ServletContext instance.");        
        bd.setValue("BeanDoc", "4.1.2");
        
        return bd;
    }

    // Create the additional bean info objects for the bean.
    private BeanInfo[] createAdditionalBeanInfo() {
        
        Vector bis = new Vector();
        BeanInfo bi = null;                
        
        //
        // Get the BeanInfo for each interface
        //          
        
        bi = null;
        try {
            bi = Introspector.getBeanInfo(javax.servlet.ServletContext.class);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }        
        
        if (bi != null) {    
            bis.addElement(bi);
        }            
        
        bi = null;
        try {
            // Get the BeanInfo for the superclass
            bi = Introspector.getBeanInfo(java.lang.Object.class);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }        
        
        if (bi != null) {    
            bis.addElement(bi);
        }                            
                        
        BeanInfo[] additionalBeanInfo = null;
        if (bis.size() > 0) {
            additionalBeanInfo = new BeanInfo[bis.size()];
            bis.copyInto(additionalBeanInfo);
        }
        
        return additionalBeanInfo;
    }

    // Create the method descriptor objects for the bean.
    private MethodDescriptor[] createMethodDescriptors() {
                
        Class[] paramTypes = null;
        int paramIndex = 0;  
        ParameterDescriptor[] paramDescriptors = null;
        ParameterDescriptor pd = null;
        
        Vector mds = new Vector();
        Method m = null;
        MethodDescriptor md = null;

           
        
        //
        // getAttribute 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("name");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getAttribute", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getAttribute");
            md.setDisplayName("getAttribute");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getAttributeNames 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getAttributeNames", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getAttributeNames");
            md.setDisplayName("getAttributeNames");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getContext 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("uripath");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getContext", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getContext");
            md.setDisplayName("getContext");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getContextPath 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getContextPath", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getContextPath");
            md.setDisplayName("getContextPath");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getInitParameter 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("name");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getInitParameter", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getInitParameter");
            md.setDisplayName("getInitParameter");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getInitParameterNames 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getInitParameterNames", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getInitParameterNames");
            md.setDisplayName("getInitParameterNames");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getMajorVersion 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getMajorVersion", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getMajorVersion");
            md.setDisplayName("getMajorVersion");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getMimeType 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("file");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getMimeType", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getMimeType");
            md.setDisplayName("getMimeType");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getMinorVersion 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getMinorVersion", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getMinorVersion");
            md.setDisplayName("getMinorVersion");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getNamedDispatcher 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("name");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getNamedDispatcher", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getNamedDispatcher");
            md.setDisplayName("getNamedDispatcher");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getRealPath 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("path");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getRealPath", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getRealPath");
            md.setDisplayName("getRealPath");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getRequestDispatcher 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("uripath");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getRequestDispatcher", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getRequestDispatcher");
            md.setDisplayName("getRequestDispatcher");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getResource 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("uripath");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getResource", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getResource");
            md.setDisplayName("getResource");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getResourceAsStream 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("uripath");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getResourceAsStream", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getResourceAsStream");
            md.setDisplayName("getResourceAsStream");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getResourcePaths 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("path");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getResourcePaths", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getResourcePaths");
            md.setDisplayName("getResourcePaths");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getServerInfo 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getServerInfo", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getServerInfo");
            md.setDisplayName("getServerInfo");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getServlet 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("name");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getServlet", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getServlet");
            md.setDisplayName("getServlet");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getServletContextName 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getServletContextName", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getServletContextName");
            md.setDisplayName("getServletContextName");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getServletNames 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getServletNames", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getServletNames");
            md.setDisplayName("getServletNames");  
            
            mds.addElement(md);
        }            
           
        
        //
        // getServlets 
        //
        paramTypes = new Class[0];
        paramDescriptors = new ParameterDescriptor[0];
        paramIndex = 0;        
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("getServlets", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("getServlets");
            md.setDisplayName("getServlets");  
            
            mds.addElement(md);
        }            
           
        
        //
        // log 
        //
        paramTypes = new Class[2];
        paramDescriptors = new ParameterDescriptor[2];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.Exception.class; 
        pd = new ParameterDescriptor();
        pd.setName("e");
        pd.setDisplayName("java.lang.Exception"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("message");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("log", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("log");
            md.setDisplayName("log");  
            
            mds.addElement(md);
        }            
           
        
        //
        // log 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("message");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("log", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("log");
            md.setDisplayName("log");  
            
            mds.addElement(md);
        }            
           
        
        //
        // log 
        //
        paramTypes = new Class[2];
        paramDescriptors = new ParameterDescriptor[2];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("message");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                        
        paramTypes[paramIndex] = java.lang.Throwable.class; 
        pd = new ParameterDescriptor();
        pd.setName("t");
        pd.setDisplayName("java.lang.Throwable"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("log", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("log");
            md.setDisplayName("log");  
            
            mds.addElement(md);
        }            
           
        
        //
        // removeAttribute 
        //
        paramTypes = new Class[1];
        paramDescriptors = new ParameterDescriptor[1];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("name");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("removeAttribute", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("removeAttribute");
            md.setDisplayName("removeAttribute");  
            
            mds.addElement(md);
        }            
           
        
        //
        // setAttribute 
        //
        paramTypes = new Class[2];
        paramDescriptors = new ParameterDescriptor[2];
        paramIndex = 0;        
                        
        paramTypes[paramIndex] = java.lang.String.class; 
        pd = new ParameterDescriptor();
        pd.setName("name");
        pd.setDisplayName("java.lang.String"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                        
        paramTypes[paramIndex] = java.lang.Object.class; 
        pd = new ParameterDescriptor();
        pd.setName("value");
        pd.setDisplayName("java.lang.Object"); 
        paramDescriptors[paramIndex] = pd;        
        paramIndex++;
                
        m = null;
        try {
            m = FilteredServletContext.class.getMethod("setAttribute", paramTypes);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        if (m != null) {
          
            md = null;
                    
            if (paramDescriptors != null) {
                md = new MethodDescriptor(m, paramDescriptors);
            }
            else {
                md = new MethodDescriptor(m);    
            }
            
            md.setName("setAttribute");
            md.setDisplayName("setAttribute");  
            
            mds.addElement(md);
        }            
               
                         
        MethodDescriptor[] methodDescriptors = 
                                new MethodDescriptor[mds.size()];
        mds.copyInto(methodDescriptors);
                
        return methodDescriptors;
    }

    // Create the property descriptor objects for the bean.
    private PropertyDescriptor[] createPropertyDescriptors() {
                           
        PropertyDescriptor[] propertyDescriptors = new PropertyDescriptor[9];
        PropertyDescriptor pd = null;
        int propertyIndex = 0;          
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("attributeNames", FilteredServletContext.class, "getAttributeNames", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("attributeNames");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("contextPath", FilteredServletContext.class, "getContextPath", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("contextPath");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("initParameterNames", FilteredServletContext.class, "getInitParameterNames", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("initParameterNames");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("majorVersion", FilteredServletContext.class, "getMajorVersion", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("majorVersion");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("minorVersion", FilteredServletContext.class, "getMinorVersion", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("minorVersion");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("serverInfo", FilteredServletContext.class, "getServerInfo", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("serverInfo");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("servletContextName", FilteredServletContext.class, "getServletContextName", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("servletContextName");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("servletNames", FilteredServletContext.class, "getServletNames", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("servletNames");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
          
        pd = null;
        try {         
            pd = new PropertyDescriptor("servlets", FilteredServletContext.class, "getServlets", null);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        if (pd != null) {    
            pd.setDisplayName("servlets");
            pd.setShortDescription("");  
                  
            propertyDescriptors[propertyIndex] = pd;
            propertyIndex++;        
        }
        
        if (propertyDescriptors.length == 0) {
            propertyDescriptors = null;    
        }
        
        return propertyDescriptors;
    }

}